@@ -4,21 +4,36 @@
 *                                                                              *
 *                              C O L E D I T . C                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 2000                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: coledit.c,v 1.1 1996/12/10 18:40:03 jrh Exp $
+* $Id: coledit.c,v 1.5 2000/12/10 15:03:02 jrh Exp $
 * $Log: coledit.c,v $
- * Revision 1.1  1996/12/10  18:40:03  jrh
- * Initial revision
- *
+* Revision 1.5  2000/12/10 15:03:02  jrh
+* Release 2.3
+*
+* Revision 1.4  1999/05/24 01:24:58  jrh
+* Release 2.2.1
+*
+* Revision 1.3  1999/02/07 21:45:41  jrh
+* Release 2.2
+*
+* Revision 1.2  1998/01/26 00:47:11  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:40:03  jrh
+* Initial revision
 *
 */
 #include<stdlib.h>
+#include<string.h>
+#include<X11/Xlib.h>
 #include<X11/Intrinsic.h>
+#include<X11/Xutil.h>
+#include<X11/Xatom.h>
 #include<Xm/Xm.h>
-#include<Xm/DialogS.h>
+#include<Xm/DrawingA.h>
 #include<Xm/Form.h>
 #include<Xm/Label.h>
 #include<Xm/LabelG.h>
@@ -31,11 +46,8 @@
 #include<Xm/Separator.h>
 #include<Xm/Text.h>
 #include<Xm/ToggleB.h>
-#ifdef __osf__
-#include<X11/GLw/GLwMDrawA.h>
-#else
-#include<GL/GLwMDrawA.h>
-#endif
+#include<X11/Xmu/StdCmap.h>
+#include<GL/glx.h>
 #include "viewmol.h"
 #include "dialog.h"
 
@@ -45,41 +57,43 @@ void GetColor(Widget, caddr_t, XmToggleButtonCallbackStruct *);
 void SetColor(Widget, float *, XmScrollBarCallbackStruct *);
 void getRGBColor(Widget, Pixel, float *, float *, float *);
 void storeColor(Pixel, struct WINDOW *, int);
+void SetSmooth(Widget, int *, XmToggleButtonCallbackStruct *);
+XVisualInfo *getVisualInfo(Display *, Visual *);
+Colormap getRGBcolormap(Display *, int, XVisualInfo *);
 
 extern Widget CreatePushButtonRow(Widget, struct PushButtonRow *, int);
-extern void GetInsWeight(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void MapBox(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern Pixel getPixel(int);
 extern void redraw(int);
 extern char *getStringResource(Widget, char *);
 extern int getIntResource(Widget, char *);
-extern void setMenuItem(int, int, int);
+extern void setMenuItem(int, int);
 extern void *getmem(size_t, size_t);
 extern void *expmem(void *, size_t, size_t);
 extern void fremem(void **);
 extern int messgb(Widget, int, char *, struct PushButtonRow *, int);
+extern Widget initShell(Widget, char *, Widget *, Widget *);
 
 extern Widget topShell;
-extern struct WINDOW windows[4];
-extern struct ATOM *atoms;
+extern struct MOLECULE *molecules;
+extern struct WINDOW windows[];
 extern struct ELEMENT *elements;
-extern struct ORBITAL *orbitals;
 extern Pixel stdcol[9], historyColor;
 extern float historyColor_rgb[4];
-extern int rgbMode, na, ne, nmodes, nhist, debug;
+extern int rgbMode, ne, debug, projectionMode;
+extern int nmolecule;
 
 static Widget dialog, red, green, blue, button[9];
 static Pixel pixelSave;
 static float rSave, gSave, bSave;
-static int window, where;
+static int window, where, smoothSave;
 
 void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
 {
-  Widget board, form, form1;
+  Widget board, form, form1, form2=(Widget)0, smoothRed, smoothGreen, smoothBlue;
   Pixmap selected, unselected;
   Pixel p;
-  Arg args[3];
   static struct PushButtonRow buttons[] = {
     { "ok", colorEditorExit, (XtPointer)TRUE, NULL },
     { "cancel", colorEditorExit, (XtPointer)FALSE, NULL }
@@ -99,49 +113,35 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
     0x01, 0x00, 0x80, 0x01, 0x00, 0x80, 0x01, 0x00, 0x80, 0x01, 0x00, 0x80,
     0x01, 0x00, 0x80, 0x01, 0x00, 0x80, 0x01, 0x00, 0x80, 0xff, 0xff, 0xff};
   float *c=NULL, r, g, b, r1, g1, b1;
+  int depth;
   char buttonName[8];
   register int i, j;
 
   /* This function creates the dialog for the color editor */
 
-  setMenuItem(VIEWER, 15, False);
+  setMenuItem(VIEWER_FOREGROUND, False);
+  setMenuItem(VIEWER_BACKGROUND, False);
   if (windows[SPECTRUM].widget != 0)
   {
-    setMenuItem(SPECTRUM, 5, False);
-    setMenuItem(SPECTRUM, 6, False);
+    setMenuItem(SPECTRUM_FOREGROUND, False);
+    setMenuItem(SPECTRUM_BACKGROUND, False);
   }
   if (windows[MO].widget != 0)
   {
-    setMenuItem(MO, 6, False);
-    setMenuItem(MO, 7, False);
+    setMenuItem(MO_FOREGROUND, False);
+    setMenuItem(MO_BACKGROUND, False);
   }
   if (windows[HISTORY].widget != 0)
   {
-    setMenuItem(HISTORY, 3, False);
-    setMenuItem(HISTORY, 4, False);
+    setMenuItem(HISTORY_ENERGY, False);
+    setMenuItem(HISTORY_GRADIENT, False);
+    setMenuItem(HISTORY_BACKGROUND, False);
   }
 
   window=(int)which & 0x7;
   where=((int)which & 0xFFFFFFF8) >> 3;
 
-  XtSetArg(args[0], XmNautoUnmanage, False);
-  XtSetArg(args[1], XmNdefaultPosition, False);
-  if (XmIsMotifWMRunning(topShell))
-    XtSetArg(args[2], XmNmwmDecorations, MWM_DECOR_RESIZEH | MWM_DECOR_TITLE);
-  else
-    XtSetArg(args[2], (char *)NULL, 0);
-  dialog=XmCreateDialogShell(topShell, "colorEditor_popup", args, XtNumber(args));
-  board=XtVaCreateWidget("colorEditor", xmFormWidgetClass, dialog,
-                         XmNautoUnmanage, False,
-                         XmNdefaultPosition, False,
-                         NULL);
-  form=XtVaCreateWidget("rowcolumn", xmRowColumnWidgetClass, board,
-                        XmNorientation, XmVERTICAL,
-                        XmNtopAttachment, XmATTACH_FORM,
-                        XmNbottomAttachment, XmATTACH_FORM,
-                        XmNleftAttachment, XmATTACH_FORM,
-                        XmNrightAttachment, XmATTACH_FORM,
-                        NULL);
+  dialog=initShell(topShell, "colorEditor", &board, &form);
 
   form1=XtVaCreateWidget("controlarea1", xmRowColumnWidgetClass, form,
                          XmNorientation, XmHORIZONTAL,
@@ -164,7 +164,7 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
                        getRGBColor(windows[window].widget, windows[window].background, &r, &g, &b);
                        pixelSave=windows[window].background;
                      }
-			   break;
+                     break;
     case FOREGROUND: if (rgbMode)
                      {
                        r=windows[window].foreground_rgb[0];
@@ -177,30 +177,31 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
                        getRGBColor(windows[window].widget, windows[window].foreground, &r, &g, &b);
                        pixelSave=windows[window].foreground;
                      }
-			   break;
+                     break;
     case GNORM:      if (rgbMode)
-		 	   {
-			     r=historyColor_rgb[0];
-			     g=historyColor_rgb[0];
-			     b=historyColor_rgb[0];
-			     pixelSave=-1;
+                     {
+                       r=historyColor_rgb[0];
+                       g=historyColor_rgb[1];
+                       b=historyColor_rgb[2];
+                       pixelSave=-1;
                      }
-			   else
-			   {
+                     else
+                     {
                        getRGBColor(windows[window].widget, historyColor, &r, &g, &b);
                        pixelSave=historyColor;
-			   }
-			   break;
+                     }
+                     break;
   }
+  XtVaGetValues(topShell, XtNdepth, &depth, NULL);
   j=WHITE;
-  for (i=0; i<8; i++)
+  for (i=0; i<=MAGENTA; i++)
   {
     sprintf(buttonName, "button%1d", i);
     p=getPixel(i);
-    selected=XCreatePixmapFromBitmapData(XtDisplay(form1), RootWindowOfScreen(XtScreen(form1)),
-                selectedData, 24, 24, stdcol[j], p, DefaultDepthOfScreen(XtScreen(form1)));
-    unselected=XCreatePixmapFromBitmapData(XtDisplay(form1), RootWindowOfScreen(XtScreen(form1)),
-                  unselectedData, 24, 24, stdcol[BLACK], p, DefaultDepthOfScreen(XtScreen(form1)));
+    selected=XCreatePixmapFromBitmapData(XtDisplay(dialog), XtWindow(dialog),
+                                         selectedData, 24, 24, stdcol[j], p, depth);
+    unselected=XCreatePixmapFromBitmapData(XtDisplay(dialog), XtWindow(dialog),
+                                           unselectedData, 24, 24, stdcol[BLACK], p, depth);
     button[i]=XtVaCreateManagedWidget(buttonName, xmToggleButtonWidgetClass, form1,
                                       XmNlabelType, XmPIXMAP,
                                       XmNlabelPixmap, unselected,
@@ -212,12 +213,40 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
     if (r1 == r && g1 == g && b1 == b) XtVaSetValues(button[i], XmNset, True, NULL);
     j=BLACK;
   }
+  if (rgbMode && where == BACKGROUND && window == VIEWER)
+  {
+    form2=XtVaCreateWidget("controlarea2", xmRowColumnWidgetClass, form,
+                           XmNorientation, XmHORIZONTAL,
+                           XmNtopAttachment, XmATTACH_WIDGET,
+                           XmNtopWidget, form1,
+                           XmNleftAttachment, XmATTACH_FORM,
+                           XmNrightAttachment, XmATTACH_FORM,
+                           NULL);
+    smoothRed=XtVaCreateManagedWidget("smoothRed", xmToggleButtonWidgetClass, form2,
+                                      XtVaTypedArg, XmNforeground, XmRString, "Red", 4,
+                                      NULL);
+    XtAddCallback(smoothRed, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+                  (XtPointer)1);
+    if (windows[window].smooth & 1) XtVaSetValues(smoothRed, XmNset, True, NULL);
+    smoothGreen=XtVaCreateManagedWidget("smoothGreen", xmToggleButtonWidgetClass, form2,
+                                        XtVaTypedArg, XmNforeground, XmRString, "Green", 6,
+                                        NULL);
+    XtAddCallback(smoothGreen, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+                  (XtPointer)2);
+    if (windows[window].smooth & 2) XtVaSetValues(smoothGreen, XmNset, True, NULL);
+    smoothBlue=XtVaCreateManagedWidget("smoothBlue", xmToggleButtonWidgetClass, form2,
+                                       XtVaTypedArg, XmNforeground, XmRString, "Blue", 5,
+                                        NULL);
+    XtAddCallback(smoothBlue, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+                  (XtPointer)4);
+    if (windows[window].smooth & 4) XtVaSetValues(smoothBlue, XmNset, True, NULL);
+  }
   rSave=r;
   gSave=g;
   bSave=b;
   red=XtVaCreateManagedWidget("red", xmScaleWidgetClass, form,
                               XmNorientation, XmHORIZONTAL,
-/*                            XtVaTypedArg, XmNtroughColor, XmRString, "Red", 4, */
+                              XtVaTypedArg, XmNtroughColor, XmRString, "Red", 4,
                               XtVaTypedArg, XmNforeground, XmRString, "Red", 4,
                               XmNvalue, (int)(1000.*r),
                               XmNminimum, 0,
@@ -231,17 +260,17 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[0];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[0];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[0];
-			   break;
+                     break;
   }
   XtAddCallback(red, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(red, XmNdragCallback, (XtCallbackProc)SetColor, c);
   green=XtVaCreateManagedWidget("green", xmScaleWidgetClass, form,
                                 XmNorientation, XmHORIZONTAL,
-/*                              XtVaTypedArg, XmNtroughColor, XmRString, "Green", 6, */
+                                XtVaTypedArg, XmNtroughColor, XmRString, "Green", 6,
                                 XtVaTypedArg, XmNforeground, XmRString, "Green", 6,
                                 XmNvalue, (int)(1000.*g),
                                 XmNminimum, 0,
@@ -255,17 +284,17 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[1];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[1];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[1];
-			   break;
+                     break;
   }
   XtAddCallback(green, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(green, XmNdragCallback, (XtCallbackProc)SetColor, c);
   blue=XtVaCreateManagedWidget("blue", xmScaleWidgetClass, form,
                                XmNorientation, XmHORIZONTAL,
-/*                             XtVaTypedArg, XmNtroughColor, XmRString, "Blue", 5, */
+                               XtVaTypedArg, XmNtroughColor, XmRString, "Blue", 5,
                                XtVaTypedArg, XmNforeground, XmRString, "Blue", 5,
                                XmNvalue, (int)(1000.*b),
                                XmNminimum, 0,
@@ -279,16 +308,18 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[2];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[2];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[2];
-			   break;
+                     break;
   }
   XtAddCallback(blue, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(blue, XmNdragCallback, (XtCallbackProc)SetColor, c);
 
   XtManageChild(form1);
+  if (rgbMode && where == BACKGROUND && window == VIEWER) XtManageChild(form2);
+
 
   CreatePushButtonRow(form, buttons, 2);
   XtAddCallback(dialog, XmNpopupCallback, (XtCallbackProc)MapBox, (XmAnyCallbackStruct *)NULL);
@@ -306,21 +337,22 @@ void colorEditorExit(Widget widget, caddr_t which, XmPushButtonCallbackStruct *d
   {
     switch (where)
     {
-	case BACKGROUND: windows[window].background=pixelSave;
+      case BACKGROUND: windows[window].background=pixelSave;
                        windows[window].background_rgb[0]=rSave;
                        windows[window].background_rgb[1]=gSave;
                        windows[window].background_rgb[2]=bSave;
-			     break;
+                       windows[window].smooth=smoothSave;
+                       break;
       case FOREGROUND: windows[window].foreground=pixelSave;
                        windows[window].foreground_rgb[0]=rSave;
                        windows[window].foreground_rgb[1]=gSave;
                        windows[window].foreground_rgb[2]=bSave;
-			     break;
+                       break;
       case GNORM:      historyColor=pixelSave;
-			     historyColor_rgb[0]=rSave;
-			     historyColor_rgb[1]=gSave;
-			     historyColor_rgb[2]=bSave;
-			     break;
+                       historyColor_rgb[0]=rSave;
+                       historyColor_rgb[1]=gSave;
+                       historyColor_rgb[2]=bSave;
+                       break;
     }
     if (!rgbMode)
     {
@@ -336,21 +368,23 @@ void colorEditorExit(Widget widget, caddr_t which, XmPushButtonCallbackStruct *d
     }
     redraw(window);
   }
-  setMenuItem(VIEWER, 15, True);
+  if (projectionMode != ORTHO) setMenuItem(VIEWER_FOREGROUND, True);
+  setMenuItem(VIEWER_BACKGROUND, True);
   if (windows[SPECTRUM].widget != 0)
   {
-    setMenuItem(SPECTRUM, 5, True);
-    setMenuItem(SPECTRUM, 6, True);
+    setMenuItem(SPECTRUM_FOREGROUND, True);
+    setMenuItem(SPECTRUM_BACKGROUND, True);
   }
   if (windows[MO].widget != 0)
   {
-    setMenuItem(MO, 6, True);
-    setMenuItem(MO, 7, True);
+    setMenuItem(MO_FOREGROUND, True);
+    setMenuItem(MO_BACKGROUND, True);
   }
   if (windows[HISTORY].widget != 0)
   {
-    setMenuItem(HISTORY, 3, True);
-    setMenuItem(HISTORY, 4, True);
+    setMenuItem(HISTORY_ENERGY, True);
+    setMenuItem(HISTORY_GRADIENT, True);
+    setMenuItem(HISTORY_BACKGROUND, True);
   }
 }
 
@@ -365,21 +399,21 @@ void GetColor(Widget widget, caddr_t which, XmToggleButtonCallbackStruct *data)
     getRGBColor(widget, stdcol[(int)which], &r, &g, &b);
     switch (where)
     {
-	case BACKGROUND: windows[window].background_rgb[0]=r;
+      case BACKGROUND: windows[window].background_rgb[0]=r;
                        windows[window].background_rgb[1]=g;
                        windows[window].background_rgb[2]=b;
                        c.pixel=windows[window].background;
-			     break;
+                       break;
       case FOREGROUND: windows[window].foreground_rgb[0]=r;
                        windows[window].foreground_rgb[1]=g;
                        windows[window].foreground_rgb[2]=b;
                        c.pixel=windows[window].foreground;
-			     break;
+                       break;
       case GNORM:      historyColor_rgb[0]=r;
                        historyColor_rgb[1]=g;
                        historyColor_rgb[2]=b;
-			     c.pixel=historyColor;
-			     break;
+                       c.pixel=historyColor;
+                       break;
     }
     XtVaSetValues(red,   XmNvalue, (int)(1000.*r), NULL);
     XtVaSetValues(green, XmNvalue, (int)(1000.*g), NULL);
@@ -413,17 +447,17 @@ void SetColor(Widget widget, float *color, XmScrollBarCallbackStruct *data)
                      c.red=(unsigned short)(windows[window].background_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(windows[window].background_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(windows[window].background_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case FOREGROUND: c.pixel=windows[window].foreground;
                      c.red=(unsigned short)(windows[window].foreground_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(windows[window].foreground_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(windows[window].foreground_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case GNORM:      c.pixel=historyColor;
                      c.red=(unsigned short)(historyColor_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(historyColor_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(historyColor_rgb[2]*65535.+0.5);
-			   break;
+                     break;
   }
   if (!rgbMode)
   {
@@ -433,7 +467,7 @@ void SetColor(Widget widget, float *color, XmScrollBarCallbackStruct *data)
                   NULL);
     XStoreColor(XtDisplay(windows[window].widget), colormap, &c);
   }
-  for (i=0; i<SKYBLUE; i++)
+  for (i=0; i<=MAGENTA; i++)
   {
     XtVaSetValues(button[i], XmNset, False, NULL);
     getRGBColor(button[i], stdcol[i], &r, &g, &b);
@@ -464,6 +498,7 @@ void getRGBColor(Widget widget, Pixel pixel, float *r, float *g, float *b)
 
 void storeColor(Pixel pixel, struct WINDOW *window, int where)
 {
+  Display *display;
   Colormap colormap;
   XColor c;
 
@@ -475,12 +510,20 @@ void storeColor(Pixel pixel, struct WINDOW *window, int where)
     case FOREGROUND: getRGBColor(window->widget, pixel, &(window->foreground_rgb[0]),
                                  &(window->foreground_rgb[1]), &(window->foreground_rgb[2]));
                      break;
-    case GNORM:      getRGBColor(window->widget, pixel, &historyColor_rgb[0], &historyColor_rgb[1],
-					   &historyColor_rgb[2]);
+    case GNORM:      display=XtDisplay(window->widget);
+                     XtVaGetValues(window->widget, XmNcolormap, &colormap, NULL);
+                     c.pixel=pixel;
+                     XQueryColor(display, colormap, &c);
+                     historyColor_rgb[0]=(float)(c.red)/65535.;
+                     historyColor_rgb[1]=(float)(c.green)/65535.;
+                     historyColor_rgb[2]=(float)(c.blue)/65535.;
                      break;
   }
   if (rgbMode) return;
-  XtVaGetValues(window->widget, XmNcolormap, &colormap, NULL);
+/*if (rgbMode)
+    colormap=DefaultColormapOfScreen(XtScreen(window->widget));
+  else*/
+    XtVaGetValues(window->widget, XmNcolormap, &colormap, NULL);
   c.flags=DoRed | DoGreen | DoBlue;
   switch (where)
   {
@@ -488,17 +531,17 @@ void storeColor(Pixel pixel, struct WINDOW *window, int where)
                      c.red=(unsigned short)(window->background_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(window->background_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(window->background_rgb[2]*65535.+0.5);
-  			   break;
+                     break;
     case FOREGROUND: c.pixel=window->foreground;
                      c.red=(unsigned short)(window->foreground_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(window->foreground_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(window->foreground_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case GNORM:      c.pixel=historyColor;
                      c.red=(unsigned short)(historyColor_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(historyColor_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(historyColor_rgb[2]*65535.+0.5);
-		  	   break;
+                     break;
   }
   XStoreColor(XtDisplay(window->widget), colormap, &c);
 }
@@ -508,7 +551,7 @@ void loadColorMap(void)
   char *colorname[9] = { "black", "white", "red", "green", "blue", "yellow",
                          "magenta", "cyan1", "SkyBlue"};
   Display *display;
-  Colormap colormap, defaultColormap;
+  Colormap colormap;
   XColor hardwarecolor, exactcolor;
   register int i;
 
@@ -516,11 +559,6 @@ void loadColorMap(void)
   XtVaGetValues(windows[VIEWER].widget,
                 XmNcolormap, &colormap,
                 NULL);
-  if (rgbMode)
-  {
-    defaultColormap=DefaultColormap(display, DefaultScreen(display));
-    if (defaultColormap != colormap) colormap=defaultColormap;
-  }
   for (i=0; i<9; i++)
   {
     if (XAllocNamedColor(display, colormap, colorname[i], &hardwarecolor,
@@ -531,105 +569,115 @@ void loadColorMap(void)
   }
 }
 
-void loadAtomColors()
+void loadAtomColors(XVisualInfo *visualInfo, Colormap colormap)
 {
   static struct PushButtonRow buttons[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL}};
+  static unsigned long *pixels;
+  static int allocated=0;
   Display *display;
-  Colormap colormap;
   XColor c;
-  XVisualInfo *vi;
   float r, g, b;
-  unsigned long *pixels, nramp;
+  unsigned long nramp;
   struct ELEMENT **e;
   char *word;
-  int l=0, reserved=0;
-  register int i, j, k, count=0;
+  size_t total=0;
+  int reserved=0, count=0, hasOrbitals=FALSE;
+  register int i, j, k;
 
-  e=(struct ELEMENT **)getmem(na, sizeof(struct ELEMENT *));
-  for (i=0; i<na; i++)
+  for (i=0; i<nmolecule; i++)
   {
-    e[i]=atoms[i].element;
+    total+=molecules[i].na;
+    if (molecules[i].basisset != NULL || molecules[i].orbitals != NULL)
+      hasOrbitals=TRUE;
   }
-  for (i=0; i<na; i++)
+  if (hasOrbitals) total+=2;
+  e=(struct ELEMENT **)getmem(total, sizeof(struct ELEMENT *));
+
+  k=0;
+  for (i=0; i<nmolecule; i++)
   {
-    for (j=0; j<i; j++)
+    for (j=0; j<molecules[i].na; j++)
+      e[k++]=molecules[i].atoms[j].element;
+  }
+  if (hasOrbitals)
+  {
+    for (i=0; i<ne; i++)
     {
-      if (e[j] && e[j] == e[i]) e[i]=(struct ELEMENT *)0;
+      if (!strcmp(elements[i].symbol, "Ps")) e[k++]=&elements[i];
+      if (!strcmp(elements[i].symbol, "Ms")) e[k++]=&elements[i];
     }
   }
-  if (orbitals != NULL)
+  for (i=0; i<total; i++)
   {
-    e=(struct ELEMENT **)expmem((void *)e, na+2, sizeof(struct ELEMENT *));
-    for (k=0; k<ne; k++)
+    if (e[i])
     {
-      if (!strcmp(elements[k].symbol, "Ps")) e[na]=&elements[k];
-      if (!strcmp(elements[k].symbol, "Ms")) e[na+1]=&elements[k];
+      for (j=i+1; j<total; j++)
+        if (e[j] && e[j] == e[i]) e[j]=(struct ELEMENT *)0;
     }
-    count=2;
-    l=2;
   }
-  for (i=0; i<na; i++)
-  {
+  for (i=0; i<total; i++)
     if (e[i]) count++;
-  }
+
   display=XtDisplay(windows[VIEWER].widget);
-  XtVaGetValues(windows[VIEWER].widget,
-                GLwNvisualInfo, &vi,
-                XmNcolormap, &colormap,
-                NULL);
   reserved=getIntResource(topShell, "reservedColors");
-  nramp=(vi->colormap_size-20-reserved)/count;
-  pixels=(unsigned long *)getmem((size_t)(nramp*count), sizeof(unsigned long));
-  while (!XAllocColorCells(display, colormap, True, NULL, 0, pixels,
-                           (unsigned long)(nramp*count-reserved)))
+  if (!allocated)
   {
-    nramp--;
-    if (nramp < 3)
+    nramp=(visualInfo->colormap_size-20-reserved)/count;
+    pixels=(unsigned long *)getmem((size_t)(nramp*count), sizeof(unsigned long));
+    while (!XAllocColorCells(display, colormap, True, NULL, 0, pixels,
+                             (unsigned long)(nramp*count-reserved)))
     {
-      word=getStringResource(topShell, "noColors");
-      messgb(topShell, 3, word, buttons, 1);
-      exit(-1);
+      nramp--;
+      if (nramp < 3)
+      {
+        word=getStringResource(topShell, "noColors");
+        messgb(topShell, 3, word, buttons, 1);
+        exit(-1);
+      }
     }
+    allocated=nramp*count-reserved;
   }
+  else
+    nramp=(allocated+reserved)/count;
   if (reserved) nramp-=reserved/count+1;
   if (debug) printf("%ld colors per atom available.\n", nramp);
-  j=0;
+  k=0;
   c.flags=DoRed | DoGreen | DoBlue;
-  for (i=0; i<na+l; i++)
+  for (i=0; i<total; i++)
   {
     if (e[i])
     {
       r=e[i]->dark[0];
       g=e[i]->dark[1];
       b=e[i]->dark[2];
-      c.pixel=pixels[j];
+      c.pixel=pixels[k];
       c.red=(unsigned short)(r*65535.+0.5);
       c.green=(unsigned short)(g*65535.+0.5);
       c.blue=(unsigned short)(b*65535.+0.5);
       XStoreColor(display, colormap, &c);
-      e[i]->colormap[0]=pixels[j++];
+      e[i]->colormap[0]=pixels[k++];
       r=(e[i]->light[0]-e[i]->dark[0])/(nramp-1);
       g=(e[i]->light[1]-e[i]->dark[1])/(nramp-1);
       b=(e[i]->light[2]-e[i]->dark[2])/(nramp-1);
-      for (k=1; k<nramp-1; k++)
+      for (j=1; j<nramp-1; j++)
       {
-        c.pixel=pixels[j];
-        c.red=(unsigned short)((e[i]->dark[0]+k*r)*65535.+0.5);
-        c.green=(unsigned short)((e[i]->dark[1]+k*g)*65535.+0.5);
-        c.blue=(unsigned short)((e[i]->dark[2]+k*b)*65535.+0.5);
+        c.pixel=pixels[k];
+        c.red=(unsigned short)((e[i]->dark[0]+j*r)*65535.+0.5);
+        c.green=(unsigned short)((e[i]->dark[1]+j*g)*65535.+0.5);
+        c.blue=(unsigned short)((e[i]->dark[2]+j*b)*65535.+0.5);
         XStoreColor(display, colormap, &c);
-        if (k == nramp/2) e[i]->colormap[1]=pixels[j];
-        j++;
+        if (j == nramp/2) e[i]->colormap[1]=pixels[k];
+        k++;
       }
       r=e[i]->light[0];
       g=e[i]->light[1];
       b=e[i]->light[2];
-      c.pixel=pixels[j];
+      c.pixel=pixels[k];
       c.red=(unsigned short)(r*65535.+0.5);
       c.green=(unsigned short)(g*65535.+0.5);
       c.blue=(unsigned short)(b*65535.+0.5);
       XStoreColor(display, colormap, &c);
-      e[i]->colormap[2]=pixels[j++];
+      e[i]->colormap[2]=pixels[k++];
     }
   }
   fremem((void **)&e);
@@ -641,24 +689,25 @@ void allocWindowColors(void)
   Display *display;
   Colormap colormap;
   Pixel pixel;
-  unsigned long ncolors=1, *pixels;
+  unsigned long ncolors=9, *pixels;
   char *word;
-  register int i=0;
+  register int i;
 
   if (rgbMode)
   {
     XtVaGetValues(windows[VIEWER].widget, XmNbackground, &pixel, NULL);
     getRGBColor(windows[VIEWER].widget, pixel, &(windows[VIEWER].background_rgb[0]),
                 &(windows[VIEWER].background_rgb[1]), &(windows[VIEWER].background_rgb[2]));
+    XtVaGetValues(windows[VIEWER].widget, XmNforeground, &pixel, NULL);
+    getRGBColor(windows[VIEWER].widget, pixel, &(windows[VIEWER].foreground_rgb[0]),
+                &(windows[VIEWER].foreground_rgb[1]), &(windows[VIEWER].foreground_rgb[2]));
+    windows[VIEWER].background_rgb[3]=1.0;
+    windows[VIEWER].foreground_rgb[3]=1.0;
   }
   else
   {
     display=XtDisplay(windows[VIEWER].widget);
     XtVaGetValues(windows[VIEWER].widget, XmNcolormap, &colormap, NULL);
-
-    if (nmodes != 0)      ncolors+=2;
-    if (nhist != 0)       ncolors+=3;
-    if (orbitals != NULL) ncolors+=2;
     pixels=(unsigned long *)getmem((size_t)ncolors, sizeof(unsigned long));
     if (!XAllocColorCells(display, colormap, True, NULL, 0, pixels, ncolors))
     {
@@ -666,25 +715,129 @@ void allocWindowColors(void)
       messgb(topShell, 3, word, buttons, 1);
       exit(-1);
     }
+    i=0;
     windows[VIEWER].background=pixels[i++];
-    if (nmodes != 0)
-    {
-      windows[SPECTRUM].background=pixels[i++];
-      windows[SPECTRUM].foreground=pixels[i++];
-    }
-    if (nhist != 0)
-    {
-      windows[HISTORY].background=pixels[i++];
-      windows[HISTORY].foreground=pixels[i++];
-	historyColor=pixels[i++];
-    }
-    if (orbitals != NULL)
-    {
-      windows[MO].background=pixels[i++];
-      windows[MO].foreground=pixels[i++];
-    }
-    free((void *)pixels);
+    windows[VIEWER].foreground=pixels[i++];
+    windows[SPECTRUM].background=pixels[i++];
+    windows[SPECTRUM].foreground=pixels[i++];
+    windows[HISTORY].background=pixels[i++];
+    windows[HISTORY].foreground=pixels[i++];
+    historyColor=pixels[i++];
+    windows[MO].background=pixels[i++];
+    windows[MO].foreground=pixels[i++];
     XtVaGetValues(windows[VIEWER].widget, XmNbackground, &pixel, NULL);
     storeColor(pixel, &windows[VIEWER], BACKGROUND);
+    XtVaGetValues(windows[VIEWER].widget, XmNforeground, &pixel, NULL);
+    storeColor(pixel, &windows[VIEWER], FOREGROUND);
+    fremem((void **)&pixels);
   }
 }
+
+void SetSmooth(Widget widget, int *which, XmToggleButtonCallbackStruct *data)
+{
+  if (data->set) windows[window].smooth |= (int)which;
+  else           windows[window].smooth &= ~(int)which;
+  redraw(window);
+}
+
+XVisualInfo *getVisualInfo(Display *display, Visual *vi)
+{
+  XVisualInfo template;
+  int items;
+
+  template.visual=vi;
+  template.visualid=vi->visualid;
+  /* leaks memory */
+  return(XGetVisualInfo(display, VisualIDMask, &template, &items));
+}
+
+Colormap getRGBcolormap(Display *dpy, int scrnum, XVisualInfo *visinfo)
+{
+/*
+ * Return an X colormap to use for OpenGL RGB-mode rendering.
+ * Input:  dpy - the X display
+ *         scrnum - the X screen number
+ *         visinfo - the XVisualInfo as returned by glXChooseVisual()
+ * Return:  an X Colormap or 0 if there's a _serious_ error.
+ */
+  Atom hp_cr_maps;
+  Status status;
+  int numCmaps, i, using_mesa;
+  XStandardColormap *standardCmaps;
+  Window root = RootWindow(dpy,scrnum);
+
+   /*
+    * First check if visinfo's visual matches the default/root visual.
+    */
+   if (visinfo->visual==DefaultVisual(dpy,scrnum)) {
+      /* use the default/root colormap */
+      return DefaultColormap( dpy, scrnum );
+   }
+
+   /*
+    * Check if we're using Mesa.
+    */
+   if (strstr(glXQueryServerString( dpy, scrnum, GLX_VERSION ), "Mesa")) {
+      using_mesa = 1;
+   }
+   else {
+      using_mesa = 0;
+   }
+
+   /*
+    * Next, if we're using Mesa and displaying on an HP with the "Color
+    * Recovery" feature and the visual is 8-bit TrueColor, search for a
+    * special colormap initialized for dithering.  Mesa will know how to
+    * dither using this colormap.
+    */
+   if (using_mesa) {
+      hp_cr_maps = XInternAtom( dpy, "_HP_RGB_SMOOTH_MAP_LIST", True );
+      if (hp_cr_maps
+          && visinfo->visual->class==TrueColor
+          && visinfo->depth==8) {
+         status = XGetRGBColormaps( dpy, root, &standardCmaps,
+                                    &numCmaps, hp_cr_maps );
+         if (status) {
+            for (i=0; i < numCmaps; i++) {
+               if (standardCmaps[i].visualid==visinfo->visual->visualid) {
+                  Colormap cmap = standardCmaps[i].colormap;
+                  XFree(standardCmaps);
+                  return cmap;
+               }
+            }
+            XFree(standardCmaps);
+         }
+      }
+   }
+
+   /*
+    * Next, try to find a standard X colormap.
+    */
+#ifndef SOLARIS_BUG
+   status = XmuLookupStandardColormap( dpy, visinfo->screen,
+                                       visinfo->visualid,
+                                       visinfo->depth,
+                                       XA_RGB_DEFAULT_MAP,
+                                       /* replace */ False,
+                                       /* retain */ True);
+   if (status == 1) {
+      status = XGetRGBColormaps( dpy, root, &standardCmaps,
+                                 &numCmaps, XA_RGB_DEFAULT_MAP);
+      if (status == 1) {
+         for (i = 0; i < numCmaps; i++) {
+            if (standardCmaps[i].visualid == visinfo->visualid) {
+               Colormap cmap = standardCmaps[i].colormap;
+               XFree(standardCmaps);
+               return cmap;
+            }
+         }
+         XFree(standardCmaps);
+      }
+   }
+#endif
+
+   /*
+    * If we get here, give up and just allocate a new colormap.
+    */
+   return XCreateColormap( dpy, root, visinfo->visual, AllocNone );
+}
