@@ -4,15 +4,26 @@
 *                                                                              *
 *                                 M E N U . H                                  *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 2000                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: menu.h,v 1.1 1996/12/10 18:45:49 jrh Exp $
+* $Id: menu.h,v 1.5 2000/12/10 15:11:11 jrh Exp $
 * $Log: menu.h,v $
- * Revision 1.1  1996/12/10  18:45:49  jrh
- * Initial revision
- *
+* Revision 1.5  2000/12/10 15:11:11  jrh
+* Release 2.3
+*
+* Revision 1.4  1999/05/24 01:28:42  jrh
+* Release 2.2.1
+*
+* Revision 1.3  1999/02/07 21:52:09  jrh
+* Release 2.2
+*
+* Revision 1.2  1998/01/26 00:48:30  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:45:49  jrh
+* Initial revision
 *
 */
 struct MenuItem
@@ -24,3 +35,10 @@ struct MenuItem
     Widget      widget;          /* the widget of the menu item */
     struct MenuItem *subitems;   /* pullright menu items, if not NULL */
 };
+
+struct MENUDATA
+{
+  struct MenuItem *submenu;
+  int window;
+  int which;
+};
