@@ -4,15 +4,26 @@
 *                                                                              *
 *                                   M O . C                                    *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 2000                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: mo.c,v 1.1 1996/12/10 18:42:19 jrh Exp $
+* $Id: mo.c,v 1.5 2000/12/10 15:12:04 jrh Exp $
 * $Log: mo.c,v $
- * Revision 1.1  1996/12/10  18:42:19  jrh
- * Initial revision
- *
+* Revision 1.5  2000/12/10 15:12:04  jrh
+* Release 2.3
+*
+* Revision 1.4  1999/05/24 01:26:35  jrh
+* Release 2.2.1
+*
+* Revision 1.3  1999/02/07 21:53:01  jrh
+* Release 2.2
+*
+* Revision 1.2  1998/01/26 00:48:45  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:42:19  jrh
+* Initial revision
 *
 */
 #include<math.h>
@@ -24,10 +35,11 @@
 
 /* Prototypes */
 double makemo(struct ATOM *, struct ORBITAL *, double, double, double, int,
-		  int, int, int);
+              int, int, int);
 void cnorm(int);
-size_t norbs(int);
-void modcmo(struct ORBITAL *, int, int, int, int, int, int);
+size_t norbs(struct MOLECULE *);
+void modcmo(struct MOLECULE *, int);
+void assignBasisName(int);
 void bfnamn(int, int *, int *);
 char *bfname(int);
 
@@ -39,8 +51,7 @@ extern void fremem(void **);
 extern char *getStringResource(Widget, char *);
 
 /* Global variables */
-extern struct BASISSET *basisset;
-extern struct ATOM *atoms;
+extern struct MOLECULE *molecules;
 extern Widget topShell;
 
 /* Local variables */
@@ -48,8 +59,8 @@ static int *atombasisnames;
 static char *basisname;
 
 double makemo(struct ATOM *atoms, struct ORBITAL *orbital, double xgrid,
-		  double ygrid, double zgrid, int na, int morb, int nbasfu,
-		  int imo)
+              double ygrid, double zgrid, int na, int morb, int nbasfu,
+              int imo)
 {
   struct BASISSET *p;
   double r, *expnt, *b;
@@ -122,14 +133,14 @@ double makemo(struct ATOM *atoms, struct ORBITAL *orbital, double xgrid,
     }
     while ((p=p->next) != NULL);
   }
-  result=0.0e0;
+  result=0.0;
   if (imo == -1)
   {
     for (j=0; j<nbasfu; j++)
     {
-      if (orbital[j].occupation != 0.0e0)
+      if (orbital[j].occupation != 0.0)
       {
-        r=0.0e0;
+        r=0.0;
         for (i=0; i<morb; i++)
           r+=orbital[j].coeff[i]*b[i];
         result+=orbital[j].occupation*r*r;
@@ -147,7 +158,7 @@ double makemo(struct ATOM *atoms, struct ORBITAL *orbital, double xgrid,
   return(result);
 }
 
-void cnorm(int na)
+void cnorm(int imol)
 {
 /*******************************************************************************
    This function normalizes the basis functions:
@@ -172,14 +183,14 @@ void cnorm(int na)
   int ang, *done;
   register int i, j, k;
 
-  fac=4.0e0*atan(1.0e0);
-  fac=pow(2.0e0*fac*fac*fac, -0.25e0);
+  fac=4.0*atan(1.0);
+  fac=pow(2.0*fac*fac*fac, -0.25);
 
-  done=(int *)getmem((size_t)na, sizeof(int));
+  done=(int *)getmem((size_t)molecules[imol].na, sizeof(int));
 
-  for (i=0; i<na; i++)
+  for (i=0; i<molecules[imol].na; i++)
   {
-    shell=atoms[i].basis;
+    shell=molecules[imol].atoms[i].basis;
     if (!done[i])
     {
       do
@@ -188,46 +199,49 @@ void cnorm(int na)
         ang=shell->ang+1;
         if (shell->nprim == 1)
         {
-          sij=1.0e0;
-          p[1]=1.0e0;
+          sij=1.0;
+          p[1]=1.0;
         }
         else
         {
-          sij=0.0e0;
+          sij=0.0;
           for (j=0; j<shell->nprim; j++)
           {
             sij+=p[2*j+1]*p[2*j+1];
             for (k=0; k<j; k++)
-              sij+=2.0e0*p[2*j+1]*p[2*k+1]*pow(4.0e0*p[2*j]*p[2*k]
-			/((p[2*j]+p[2*k])*(p[2*j]+p[2*k])), 0.5e0*ang+0.25e0);
+              sij+=2.0*p[2*j+1]*p[2*k+1]*pow(4.0*p[2*j]*p[2*k]
+                  /((p[2*j]+p[2*k])*(p[2*j]+p[2*k])), 0.5*ang+0.25);
           }
-          sij=1.0e0/sqrt(sij);
+          sij=1.0/sqrt(sij);
         }
         for (j=0; j<shell->nprim; j++)
         {
-          nii=fac*pow(2.0e0, (double)ang)*pow(p[2*j], 0.5e0*ang+0.25e0);
+          nii=fac*pow(2.0, (double)ang)*pow(p[2*j], 0.5*ang+0.25);
           p[2*j+1]*=nii*sij;
         }
         shell=shell->next;
       } while (shell != NULL);
       done[i]=TRUE;
-      for (j=0; j<na; j++)
-        if (atoms[i].basis == atoms[j].basis) done[j]=TRUE;
+      for (j=0; j<molecules[imol].na; j++)
+      {
+        if (molecules[imol].atoms[i].basis == molecules[imol].atoms[j].basis)
+          done[j]=TRUE;
+      }
     }
   }
   fremem((void **)&done);
 }
 
-size_t norbs(int na)
+size_t norbs(struct MOLECULE *mol)
 {                                   /* determines the total number  */
   struct BASISSET *shell;           /* of cartesian basis functions */
   register size_t nmo;
   register int atom;
 
   nmo=0;
-  for (atom=0; atom<na; atom++)
+  for (atom=0; atom<mol->na; atom++)
   {
-    shell=atoms[atom].basis;
+    shell=mol->atoms[atom].basis;
     do
     {
       nmo+=(shell->ang+1)*(shell->ang+2)/2;
@@ -237,8 +251,7 @@ size_t norbs(int na)
   return(nmo);
 }
 
-void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
-            int na, int nbasfu, int gaussian)
+void modcmo(struct MOLECULE *mol, int gaussian)
 {
 /*******************************************************************************
 
@@ -267,9 +280,9 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
   way:
   d functions: d(xx), d(yy), d(zz), d(xy), d(xz), d(yz)
   f functions: f(xxx), f(yyy), f(zzz), f(xxy), f(xxz), f(yyx), f(yyz),
-		   f(zzx), f(zzy), f(xyz)
+               f(zzx), f(zzy), f(xyz)
   g functions: g(xxxx), g(yyyy), g(zzzz), g(xxxy), g(xxxz), g(yyyx),
-		   g(yyyz), g(zzzx), g(zzzy), g(xxyy), g(xxzz), g(yyzz),
+               g(yyyz), g(zzzx), g(zzzy), g(xxyy), g(xxzz), g(yyzz),
                g(xxyz), g(yyxz), g(zzxy)
 
 *******************************************************************************/
@@ -277,6 +290,7 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
   static struct PushButtonRow buttons[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL},
                                            {"continue", GetMessageBoxButton, (XtPointer)1, NULL}};
 
+  struct ORBITAL *orbital;
   struct BASISSET *shell;
   double *newmo, *c, help;
   int lmax=(-1), fproblem=FALSE, gproblem=FALSE;
@@ -284,16 +298,17 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
   size_t ntotal;
   register int mo, atom, i, j;
 
-  ntotal=norbs(na);
-  newmo=(double *)getmem(ntotal*nbasfu, sizeof(double));
+  orbital=mol->orbitals;
+  ntotal=norbs(mol);
+  newmo=(double *)getmem(ntotal*mol->nbasfu, sizeof(double));
   j=0;
-  for (mo=0; mo<nbasfu; mo++)
+  for (mo=0; mo<mol->nbasfu; mo++)
   {
     i=0;
     c=&newmo[j];
-    for (atom=0; atom<na; atom++)
+    for (atom=0; atom<mol->na; atom++)
     {
-      shell=atoms[atom].basis;
+      shell=mol->atoms[atom].basis;
       do
       {
         if (shell->ang > lmax) lmax=shell->ang;
@@ -309,7 +324,7 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                    i+=3;
                    j+=3;
                    break;
-          case 2:  if (pured)
+          case 2:  if (mol->pured)
                    {
                      if (gaussian)
                      {
@@ -356,7 +371,7 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                    }
                    j+=6;
                    break;
-          case 3:  if (puref)
+          case 3:  if (mol->puref)
                    {
                      if (gaussian)
                      {
@@ -417,8 +432,8 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                    }
                    j+=10;
                    break;
-          case 4:  if (pureg)
-			 {
+          case 4:  if (mol->pureg)
+                   {
                      if (gaussian) gproblem=TRUE;
                      else
                      {
@@ -451,16 +466,16 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                                   -0.5*sqrt(3./2.)*orbital[mo].coeff[i+5]);
                        newmo[j+14]=sqrt(3./7.)*orbital[mo].coeff[i+3];
                      }
-	 		   i+=9;
-			 }
-			 else
-			 {
+                      i+=9;
+                   }
+                   else
+                   {
                      if (gaussian) gproblem=TRUE;
                      else
                      {
                        newmo[j]=0.125*sqrt(3./35.)*orbital[mo].coeff[i]
                                +0.25/sqrt(21.)*orbital[mo].coeff[i+4]
-		                   +0.125/sqrt(3.)*orbital[mo].coeff[i+8]
+                               +0.125/sqrt(3.)*orbital[mo].coeff[i+8]
                                -1./(6.*sqrt(21.))*orbital[mo].coeff[i+9]
                                +1./(6.*sqrt(7.))*orbital[mo].coeff[i+13]
                                +1./(3.*sqrt(105.))*orbital[mo].coeff[i+14];
@@ -512,10 +527,10 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                        newmo[j+14]=sqrt(3./7.)*orbital[mo].coeff[i+3]
                                   +1./(3.*sqrt(7.))*orbital[mo].coeff[i+12];
                      }
-	 		   i+=15;
-			 }
+                      i+=15;
+                   }
                    j+=15;
-			 break;
+                   break;
           default: break;
         }
         shell=shell->next;
@@ -541,17 +556,46 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
     fremem((void **)&orbital);
     return;
   }
+  if (lmax > LMAX)
+  {
+    strcpy(line, "I cannot handle ");
+    for (i=LMAX+1; i<lmax; i++)
+    {
+      sprintf(l, "%c, ", i+'c');
+      strcat(line, l);
+    }
+    sprintf(l, "%c-", lmax+'c');
+    if (LMAX+1 != lmax)
+    {
+      if (lmax-LMAX == 2)
+        sprintf(line+strlen(line)-2, "and ");
+      else
+        strcat(line, "and ");
+    }
+    strcat(line, l);
+    strcat(line, "functions.");
+/*  if (messgb(&mx, &my, &one, line, "Continue|Exit programme") == 2) exit(-1);  */
+  }
+}
 
+void assignBasisName(int imol)
+{
+  struct BASISSET *shell;
+  size_t ntotal;
+  char *p;
+  register int atom, j;
+
+  ntotal=norbs(&molecules[imol]);
   if (basisname == NULL)
     basisname=(char *)getmem((size_t)ntotal, 8*sizeof(char));
   if (atombasisnames == NULL)
-    atombasisnames=(int *)getmem((size_t)na, 2*sizeof(int));
+    atombasisnames=(int *)getmem((size_t)molecules[imol].na, 2*sizeof(int));
   j=0;
   p=basisname;
-  for (atom=0; atom<na; atom++)              /* assign a name for each basis */
-  {                                          /* function                     */
+  for (atom=0; atom<molecules[imol].na; atom++) /* assign a name for each basis */
+  {                                             /* function                     */
     atombasisnames[2*atom]=j;
-    shell=atoms[atom].basis;
+    shell=molecules[imol].atoms[atom].basis;
     do
     {
       switch (shell->ang)
@@ -603,34 +647,14 @@ void modcmo(struct ORBITAL *orbital, int pured, int puref, int pureg,
                 strcpy(p+96, "g(xxyz)");
                 strcpy(p+104,"g(yyxz)");
                 strcpy(p+112,"g(zzxy)");
-		    p+=120;
-		    j+=15;
-		    break;
+                p+=120;
+                j+=15;
+                break;
       }
       shell=shell->next;
     } while (shell != NULL);
     atombasisnames[2*atom+1]=j-1;
   }
-  if (lmax > LMAX)
-  {
-    strcpy(line, "I cannot handle ");
-    for (i=LMAX+1; i<lmax; i++)
-    {
-      sprintf(l, "%c, ", i+'c');
-      strcat(line, l);
-    }
-    sprintf(l, "%c-", lmax+'c');
-    if (LMAX+1 != lmax)
-    {
-      if (lmax-LMAX == 2)
-        sprintf(line+strlen(line)-2, "and ");
-      else
-        strcat(line, "and ");
-    }
-    strcat(line, l);
-    strcat(line, "functions.");
-/*  if (messgb(&mx, &my, &one, line, "Continue|Exit programme") == 2) exit(-1);  */
-  }
 }
 
 void bfnamn(int atom, int *start, int *end)
