@@ -4,15 +4,26 @@
 *                                                                              *
 *                               W I N D O W . C                                *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 2000                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: window.c,v 1.1 1996/12/10 18:44:28 jrh Exp $
+* $Id: window.c,v 1.5 2000/12/10 15:19:04 jrh Exp $
 * $Log: window.c,v $
- * Revision 1.1  1996/12/10  18:44:28  jrh
- * Initial revision
- *
+* Revision 1.5  2000/12/10 15:19:04  jrh
+* Release 2.3
+*
+* Revision 1.4  1999/05/24 01:27:59  jrh
+* Release 2.2.1
+*
+* Revision 1.3  1999/02/07 21:59:50  jrh
+* Release 2.2
+*
+* Revision 1.2  1998/01/26 00:49:49  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:44:28  jrh
+* Initial revision
 *
 */
 #include<X11/Xlib.h>
@@ -23,4 +34,12 @@
 void setWindowTitle(Widget widget, char *title)
 {
   XtVaSetValues(XtParent(widget), XmNtitle, title, NULL);
+
+/*XTextProperty text;
+
+  if (XStringListToTextProperty(&title, 1, &text))
+  {
+    XSetWMName(XtDisplay(widget), XtWindow(widget), &text);
+    XFlush(XtDisplay(widget));
+  } */
 }
